"""
Pydantic Models for CMDB - router/ospf

Runtime validation models for router/ospf configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

class OspfOspfInterfaceNetworkTypeEnum(str, Enum):
    """Allowed values for network_type field in ospf-interface."""
    BROADCAST = "broadcast"
    NON_BROADCAST = "non-broadcast"
    POINT_TO_POINT = "point-to-point"
    POINT_TO_MULTIPOINT = "point-to-multipoint"
    POINT_TO_MULTIPOINT_NON_BROADCAST = "point-to-multipoint-non-broadcast"

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class OspfSummaryAddress(BaseModel):
    """
    Child table model for summary-address.
    
    IP address summary configuration.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Summary address entry ID.")    
    prefix: str = Field(default="0.0.0.0 0.0.0.0", description="Prefix.")    
    tag: int | None = Field(ge=0, le=4294967295, default=0, description="Tag value.")    
    advertise: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable advertise status.")
class OspfRedistribute(BaseModel):
    """
    Child table model for redistribute.
    
    Redistribute configuration.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str = Field(max_length=35, description="Redistribute name.")    
    status: Literal["enable", "disable"] | None = Field(default="disable", description="Status.")    
    metric: int | None = Field(ge=0, le=16777214, default=0, description="Redistribute metric setting.")    
    routemap: str | None = Field(max_length=35, default=None, description="Route map name.")  # datasource: ['router.route-map.name']    
    metric_type: Literal["1", "2"] | None = Field(default="2", description="Metric type.")    
    tag: int | None = Field(ge=0, le=4294967295, default=0, description="Tag value.")
class OspfPassiveInterface(BaseModel):
    """
    Child table model for passive-interface.
    
    Passive interface configuration.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str = Field(max_length=79, description="Passive interface name.")  # datasource: ['system.interface.name']
class OspfOspfInterfaceMd5Keys(BaseModel):
    """
    Child table model for ospf-interface.md5-keys.
    
    MD5 key.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int = Field(ge=1, le=255, default=0, serialization_alias="id", description="Key ID (1 - 255).")    
    key_string: Any = Field(max_length=16, description="Password for the key.")
class OspfOspfInterface(BaseModel):
    """
    Child table model for ospf-interface.
    
    OSPF interface configuration.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str | None = Field(max_length=35, default=None, description="Interface entry name.")    
    comments: str | None = Field(max_length=255, default=None, description="Comment.")    
    interface: str = Field(max_length=15, description="Configuration interface name.")  # datasource: ['system.interface.name']    
    ip: str | None = Field(default="0.0.0.0", description="IP address.")    
    linkdown_fast_failover: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable fast link failover.")    
    authentication: Literal["none", "text", "message-digest"] | None = Field(default="none", description="Authentication type.")    
    authentication_key: Any = Field(max_length=8, default=None, description="Authentication key.")    
    keychain: str | None = Field(max_length=35, default=None, description="Message-digest key-chain name.")  # datasource: ['router.key-chain.name']    
    prefix_length: int | None = Field(ge=0, le=32, default=0, description="Prefix length.")    
    retransmit_interval: int | None = Field(ge=1, le=65535, default=5, description="Retransmit interval.")    
    transmit_delay: int | None = Field(ge=1, le=65535, default=1, description="Transmit delay.")    
    cost: int | None = Field(ge=0, le=65535, default=0, description="Cost of the interface, value range from 0 to 65535, 0 means auto-cost.")    
    priority: int | None = Field(ge=0, le=255, default=1, description="Priority.")    
    dead_interval: int | None = Field(ge=0, le=65535, default=0, description="Dead interval.")    
    hello_interval: int | None = Field(ge=0, le=65535, default=0, description="Hello interval.")    
    hello_multiplier: int | None = Field(ge=3, le=10, default=0, description="Number of hello packets within dead interval.")    
    database_filter_out: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable control of flooding out LSAs.")    
    mtu: int | None = Field(ge=576, le=65535, default=0, description="MTU for database description packets.")    
    mtu_ignore: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable ignore MTU.")    
    network_type: OspfOspfInterfaceNetworkTypeEnum | None = Field(default=OspfOspfInterfaceNetworkTypeEnum.BROADCAST, description="Network type.")    
    bfd: Literal["global", "enable", "disable"] | None = Field(default="global", description="Bidirectional Forwarding Detection (BFD).")    
    status: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable status.")    
    resync_timeout: int | None = Field(ge=1, le=3600, default=40, description="Graceful restart neighbor resynchronization timeout.")    
    md5_keys: list[OspfOspfInterfaceMd5Keys] = Field(default_factory=list, description="MD5 key.")
class OspfNetwork(BaseModel):
    """
    Child table model for network.
    
    OSPF network configuration.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Network entry ID.")    
    prefix: str = Field(default="0.0.0.0 0.0.0.0", description="Prefix.")    
    area: str = Field(default="0.0.0.0", description="Attach the network to area.")    
    comments: str | None = Field(max_length=255, default=None, description="Comment.")
class OspfNeighbor(BaseModel):
    """
    Child table model for neighbor.
    
    OSPF neighbor configuration are used when OSPF runs on non-broadcast media.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Neighbor entry ID.")    
    ip: str = Field(default="0.0.0.0", description="Interface IP address of the neighbor.")    
    poll_interval: int | None = Field(ge=1, le=65535, default=10, description="Poll interval time in seconds.")    
    cost: int | None = Field(ge=0, le=65535, default=0, description="Cost of the interface, value range from 0 to 65535, 0 means auto-cost.")    
    priority: int | None = Field(ge=0, le=255, default=1, description="Priority.")
class OspfDistributeList(BaseModel):
    """
    Child table model for distribute-list.
    
    Distribute list configuration.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Distribute list entry ID.")    
    access_list: str = Field(max_length=35, description="Access list name.")  # datasource: ['router.access-list.name']    
    protocol: Literal["connected", "static", "rip"] = Field(default="connected", description="Protocol type.")
class OspfAreaVirtualLinkMd5Keys(BaseModel):
    """
    Child table model for area.virtual-link.md5-keys.
    
    MD5 key.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int = Field(ge=1, le=255, default=0, serialization_alias="id", description="Key ID (1 - 255).")    
    key_string: Any = Field(max_length=16, description="Password for the key.")
class OspfAreaVirtualLink(BaseModel):
    """
    Child table model for area.virtual-link.
    
    OSPF virtual link configuration.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str | None = Field(max_length=35, default=None, description="Virtual link entry name.")    
    authentication: Literal["none", "text", "message-digest"] | None = Field(default="none", description="Authentication type.")    
    authentication_key: Any = Field(max_length=8, default=None, description="Authentication key.")    
    keychain: str | None = Field(max_length=35, default=None, description="Message-digest key-chain name.")  # datasource: ['router.key-chain.name']    
    dead_interval: int | None = Field(ge=1, le=65535, default=40, description="Dead interval.")    
    hello_interval: int | None = Field(ge=1, le=65535, default=10, description="Hello interval.")    
    retransmit_interval: int | None = Field(ge=1, le=65535, default=5, description="Retransmit interval.")    
    transmit_delay: int | None = Field(ge=1, le=65535, default=1, description="Transmit delay.")    
    peer: str = Field(default="0.0.0.0", description="Peer IP.")    
    md5_keys: list[OspfAreaVirtualLinkMd5Keys] = Field(default_factory=list, description="MD5 key.")
class OspfAreaRange(BaseModel):
    """
    Child table model for area.range.
    
    OSPF area range configuration.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Range entry ID.")    
    prefix: Any = Field(default="0.0.0.0 0.0.0.0", description="Prefix.")    
    advertise: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable advertise status.")    
    substitute: Any = Field(default="0.0.0.0 0.0.0.0", description="Substitute prefix.")    
    substitute_status: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable substitute status.")
class OspfAreaFilterList(BaseModel):
    """
    Child table model for area.filter-list.
    
    OSPF area filter-list configuration.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Filter list entry ID.")    
    list_: str = Field(max_length=35, serialization_alias="list", description="Access-list or prefix-list name.")  # datasource: ['router.access-list.name', 'router.prefix-list.name']    
    direction: Literal["in", "out"] = Field(default="out", description="Direction.")
class OspfArea(BaseModel):
    """
    Child table model for area.
    
    OSPF area configuration.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: str | None = Field(default="0.0.0.0", serialization_alias="id", description="Area entry IP address.")    
    shortcut: Literal["disable", "enable", "default"] | None = Field(default="disable", description="Enable/disable shortcut option.")    
    authentication: Literal["none", "text", "message-digest"] | None = Field(default="none", description="Authentication type.")    
    default_cost: int | None = Field(ge=0, le=4294967295, default=10, description="Summary default cost of stub or NSSA area.")    
    nssa_translator_role: Literal["candidate", "never", "always"] | None = Field(default="candidate", description="NSSA translator role type.")    
    stub_type: Literal["no-summary", "summary"] | None = Field(default="summary", description="Stub summary setting.")    
    type_: Literal["regular", "nssa", "stub"] | None = Field(default="regular", serialization_alias="type", description="Area type setting.")    
    nssa_default_information_originate: Literal["enable", "always", "disable"] | None = Field(default="disable", description="Redistribute, advertise, or do not originate Type-7 default route into NSSA area.")    
    nssa_default_information_originate_metric: int | None = Field(ge=0, le=16777214, default=10, description="OSPF default metric.")    
    nssa_default_information_originate_metric_type: Literal["1", "2"] | None = Field(default="2", description="OSPF metric type for default routes.")    
    nssa_redistribution: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable redistribute into NSSA area.")    
    comments: str | None = Field(max_length=255, default=None, description="Comment.")    
    range_: list[OspfAreaRange] = Field(default_factory=list, serialization_alias="range", description="OSPF area range configuration.")    
    virtual_link: list[OspfAreaVirtualLink] = Field(default_factory=list, description="OSPF virtual link configuration.")    
    filter_list: list[OspfAreaFilterList] = Field(default_factory=list, description="OSPF area filter-list configuration.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class OspfAbrTypeEnum(str, Enum):
    """Allowed values for abr_type field."""
    CISCO = "cisco"
    IBM = "ibm"
    SHORTCUT = "shortcut"
    STANDARD = "standard"


# ============================================================================
# Main Model
# ============================================================================

class OspfModel(BaseModel):
    """
    Pydantic model for router/ospf configuration.
    
    Configure OSPF.
    
    Validation Rules:        - abr_type: pattern=        - auto_cost_ref_bandwidth: min=1 max=1000000 pattern=        - distance_external: min=1 max=255 pattern=        - distance_inter_area: min=1 max=255 pattern=        - distance_intra_area: min=1 max=255 pattern=        - database_overflow: pattern=        - database_overflow_max_lsas: min=0 max=4294967295 pattern=        - database_overflow_time_to_recover: min=0 max=65535 pattern=        - default_information_originate: pattern=        - default_information_metric: min=1 max=16777214 pattern=        - default_information_metric_type: pattern=        - default_information_route_map: max_length=35 pattern=        - default_metric: min=1 max=16777214 pattern=        - distance: min=1 max=255 pattern=        - lsa_refresh_interval: min=0 max=5 pattern=        - rfc1583_compatible: pattern=        - router_id: pattern=        - spf_timers: pattern=        - bfd: pattern=        - log_neighbour_changes: pattern=        - distribute_list_in: max_length=35 pattern=        - distribute_route_map_in: max_length=35 pattern=        - restart_mode: pattern=        - restart_period: min=1 max=3600 pattern=        - restart_on_topology_change: pattern=        - area: pattern=        - ospf_interface: pattern=        - network: pattern=        - neighbor: pattern=        - passive_interface: pattern=        - summary_address: pattern=        - distribute_list: pattern=        - redistribute: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    abr_type: OspfAbrTypeEnum | None = Field(default=OspfAbrTypeEnum.STANDARD, description="Area border router type.")    
    auto_cost_ref_bandwidth: int | None = Field(ge=1, le=1000000, default=1000, description="Reference bandwidth in terms of megabits per second.")    
    distance_external: int | None = Field(ge=1, le=255, default=110, description="Administrative external distance.")    
    distance_inter_area: int | None = Field(ge=1, le=255, default=110, description="Administrative inter-area distance.")    
    distance_intra_area: int | None = Field(ge=1, le=255, default=110, description="Administrative intra-area distance.")    
    database_overflow: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable database overflow.")    
    database_overflow_max_lsas: int | None = Field(ge=0, le=4294967295, default=10000, description="Database overflow maximum LSAs.")    
    database_overflow_time_to_recover: int | None = Field(ge=0, le=65535, default=300, description="Database overflow time to recover (sec).")    
    default_information_originate: Literal["enable", "always", "disable"] | None = Field(default="disable", description="Enable/disable generation of default route.")    
    default_information_metric: int | None = Field(ge=1, le=16777214, default=10, description="Default information metric.")    
    default_information_metric_type: Literal["1", "2"] | None = Field(default="2", description="Default information metric type.")    
    default_information_route_map: str | None = Field(max_length=35, default=None, description="Default information route map.")  # datasource: ['router.route-map.name']    
    default_metric: int | None = Field(ge=1, le=16777214, default=10, description="Default metric of redistribute routes.")    
    distance: int | None = Field(ge=1, le=255, default=110, description="Distance of the route.")    
    lsa_refresh_interval: int | None = Field(ge=0, le=5, default=5, description="The minimal OSPF LSA update time interval")    
    rfc1583_compatible: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable RFC1583 compatibility.")    
    router_id: str = Field(default="0.0.0.0", description="Router ID.")    
    spf_timers: str | None = Field(default=None, description="SPF calculation frequency.")    
    bfd: Literal["enable", "disable"] | None = Field(default="disable", description="Bidirectional Forwarding Detection (BFD).")    
    log_neighbour_changes: Literal["enable", "disable"] | None = Field(default="enable", description="Log of OSPF neighbor changes.")    
    distribute_list_in: str | None = Field(max_length=35, default=None, description="Filter incoming routes.")  # datasource: ['router.access-list.name', 'router.prefix-list.name']    
    distribute_route_map_in: str | None = Field(max_length=35, default=None, description="Filter incoming external routes by route-map.")  # datasource: ['router.route-map.name']    
    restart_mode: Literal["none", "lls", "graceful-restart"] | None = Field(default="none", description="OSPF restart mode (graceful or LLS).")    
    restart_period: int | None = Field(ge=1, le=3600, default=120, description="Graceful restart period.")    
    restart_on_topology_change: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable continuing graceful restart upon topology change.")    
    area: list[OspfArea] = Field(default_factory=list, description="OSPF area configuration.")    
    ospf_interface: list[OspfOspfInterface] = Field(default_factory=list, description="OSPF interface configuration.")    
    network: list[OspfNetwork] = Field(default_factory=list, description="OSPF network configuration.")    
    neighbor: list[OspfNeighbor] = Field(default_factory=list, description="OSPF neighbor configuration are used when OSPF runs on non-broadcast media.")    
    passive_interface: list[OspfPassiveInterface] = Field(default_factory=list, description="Passive interface configuration.")    
    summary_address: list[OspfSummaryAddress] = Field(default_factory=list, description="IP address summary configuration.")    
    distribute_list: list[OspfDistributeList] = Field(default_factory=list, description="Distribute list configuration.")    
    redistribute: list[OspfRedistribute] = Field(default_factory=list, description="Redistribute configuration.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('default_information_route_map')
    @classmethod
    def validate_default_information_route_map(cls, v: Any) -> Any:
        """
        Validate default_information_route_map field.
        
        Datasource: ['router.route-map.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('distribute_list_in')
    @classmethod
    def validate_distribute_list_in(cls, v: Any) -> Any:
        """
        Validate distribute_list_in field.
        
        Datasource: ['router.access-list.name', 'router.prefix-list.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('distribute_route_map_in')
    @classmethod
    def validate_distribute_route_map_in(cls, v: Any) -> Any:
        """
        Validate distribute_route_map_in field.
        
        Datasource: ['router.route-map.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "OspfModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_default_information_route_map_references(self, client: Any) -> list[str]:
        """
        Validate default_information_route_map references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - router/route-map        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = OspfModel(
            ...     default_information_route_map="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_default_information_route_map_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.router.ospf.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "default_information_route_map", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.router.route_map.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Default-Information-Route-Map '{value}' not found in "
                "router/route-map"
            )        
        return errors    
    async def validate_distribute_list_in_references(self, client: Any) -> list[str]:
        """
        Validate distribute_list_in references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - router/access-list        - router/prefix-list        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = OspfModel(
            ...     distribute_list_in="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_distribute_list_in_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.router.ospf.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "distribute_list_in", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.router.access_list.exists(value):
            found = True
        elif await client.api.cmdb.router.prefix_list.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Distribute-List-In '{value}' not found in "
                "router/access-list or router/prefix-list"
            )        
        return errors    
    async def validate_distribute_route_map_in_references(self, client: Any) -> list[str]:
        """
        Validate distribute_route_map_in references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - router/route-map        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = OspfModel(
            ...     distribute_route_map_in="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_distribute_route_map_in_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.router.ospf.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "distribute_route_map_in", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.router.route_map.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Distribute-Route-Map-In '{value}' not found in "
                "router/route-map"
            )        
        return errors    
    async def validate_ospf_interface_references(self, client: Any) -> list[str]:
        """
        Validate ospf_interface references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - router/key-chain        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = OspfModel(
            ...     ospf_interface=[{"keychain": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_ospf_interface_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.router.ospf.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "ospf_interface", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("keychain")
            else:
                value = getattr(item, "keychain", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.router.key_chain.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Ospf-Interface '{value}' not found in "
                    "router/key-chain"
                )        
        return errors    
    async def validate_passive_interface_references(self, client: Any) -> list[str]:
        """
        Validate passive_interface references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/interface        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = OspfModel(
            ...     passive_interface=[{"name": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_passive_interface_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.router.ospf.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "passive_interface", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("name")
            else:
                value = getattr(item, "name", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.system.interface.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Passive-Interface '{value}' not found in "
                    "system/interface"
                )        
        return errors    
    async def validate_distribute_list_references(self, client: Any) -> list[str]:
        """
        Validate distribute_list references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - router/access-list        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = OspfModel(
            ...     distribute_list=[{"access-list": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_distribute_list_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.router.ospf.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "distribute_list", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("access-list")
            else:
                value = getattr(item, "access-list", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.router.access_list.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Distribute-List '{value}' not found in "
                    "router/access-list"
                )        
        return errors    
    async def validate_redistribute_references(self, client: Any) -> list[str]:
        """
        Validate redistribute references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - router/route-map        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = OspfModel(
            ...     redistribute=[{"routemap": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_redistribute_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.router.ospf.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "redistribute", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("routemap")
            else:
                value = getattr(item, "routemap", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.router.route_map.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Redistribute '{value}' not found in "
                    "router/route-map"
                )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_default_information_route_map_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_distribute_list_in_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_distribute_route_map_in_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_ospf_interface_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_passive_interface_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_distribute_list_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_redistribute_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "OspfModel",    "OspfArea",    "OspfArea.Range",    "OspfArea.VirtualLink",    "OspfArea.VirtualLink.Md5Keys",    "OspfArea.FilterList",    "OspfOspfInterface",    "OspfOspfInterface.Md5Keys",    "OspfNetwork",    "OspfNeighbor",    "OspfPassiveInterface",    "OspfSummaryAddress",    "OspfDistributeList",    "OspfRedistribute",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.2
# Generated: 2026-01-21T16:24:00.519460Z
# ============================================================================


"""
Pydantic Models for MONITOR - vpn_certificate/csr/generate

Runtime validation models for vpn_certificate/csr/generate configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class GenerateKeysizeEnum(str, Enum):
    """Allowed values for keysize field."""
    V_1024 = "1024"
    V_1536 = "1536"
    V_2048 = "2048"
    V_4096 = "4096"


# ============================================================================
# Main Model
# ============================================================================

class GenerateModel(BaseModel):
    """
    Pydantic model for vpn_certificate/csr/generate configuration.
    
    Generate a certificate signing request (CSR) and a private key. The CSR can be retrieved / downloaded from CLI, GUI and REST API.
    
    Validation Rules:        - certname: pattern=        - subject: pattern=        - keytype: pattern=        - keysize: pattern=        - curvename: pattern=        - orgunits: pattern=        - org: pattern=        - city: pattern=        - state: pattern=        - countrycode: pattern=        - email: pattern=        - subject_alt_name: pattern=        - password: pattern=        - scep_url: pattern=        - scep_password: pattern=        - scope: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    certname: str = Field()    
    subject: str = Field()    
    keytype: Literal["rsa", "ec"] = Field()    
    keysize: GenerateKeysizeEnum | None = Field(default=None)    
    curvename: Literal["secp256r1", "secp384r1", "secp521r1"] | None = Field(default=None)    
    orgunits: Any = Field(default=None)    
    org: str | None = Field(default=None)    
    city: str | None = Field(default=None)    
    state: str | None = Field(default=None)    
    countrycode: str | None = Field(default=None)    
    email: str | None = Field(default=None)    
    subject_alt_name: str | None = Field(default=None)    
    password: str | None = Field(default=None)    
    scep_url: str | None = Field(default=None)    
    scep_password: str | None = Field(default=None)    
    scope: Literal["vdom", "global"] | None = Field(default=None)    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "GenerateModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "GenerateModel",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-21T16:24:02.769819Z
# ============================================================================


"""Type stubs for EXTENSION_CONTROLLER category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .dataplan import Dataplan
    from .extender import Extender
    from .extender_profile import ExtenderProfile
    from .extender_vap import ExtenderVap
    from .fortigate import Fortigate
    from .fortigate_profile import FortigateProfile

__all__ = [
    "Dataplan",
    "Extender",
    "ExtenderProfile",
    "ExtenderVap",
    "Fortigate",
    "FortigateProfile",
    "ExtensionController",
]


class ExtensionController:
    """EXTENSION_CONTROLLER API category.
    
    All endpoints return FortiObject instances with:
    - Attribute access: response.field
    - Dictionary access: response["field"]
    - Convert to dict: response.dict or response.json
    """
    
    dataplan: Dataplan
    extender: Extender
    extender_profile: ExtenderProfile
    extender_vap: ExtenderVap
    fortigate: Fortigate
    fortigate_profile: FortigateProfile

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize extension_controller category with HTTP client."""
        ...



""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: wireless_controller/hotspot20/hs_profile
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
    overload,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class HsProfileOsuproviderItem(TypedDict, total=False):
    """Nested item for osu-provider field."""
    name: str


class HsProfilePayload(TypedDict, total=False):
    """Payload type for HsProfile operations."""
    name: str
    release: int
    access_network_type: Literal["private-network", "private-network-with-guest-access", "chargeable-public-network", "free-public-network", "personal-device-network", "emergency-services-only-network", "test-or-experimental", "wildcard"]
    access_network_internet: Literal["enable", "disable"]
    access_network_asra: Literal["enable", "disable"]
    access_network_esr: Literal["enable", "disable"]
    access_network_uesa: Literal["enable", "disable"]
    venue_group: Literal["unspecified", "assembly", "business", "educational", "factory", "institutional", "mercantile", "residential", "storage", "utility", "vehicular", "outdoor"]
    venue_type: Literal["unspecified", "arena", "stadium", "passenger-terminal", "amphitheater", "amusement-park", "place-of-worship", "convention-center", "library", "museum", "restaurant", "theater", "bar", "coffee-shop", "zoo-or-aquarium", "emergency-center", "doctor-office", "bank", "fire-station", "police-station", "post-office", "professional-office", "research-facility", "attorney-office", "primary-school", "secondary-school", "university-or-college", "factory", "hospital", "long-term-care-facility", "rehab-center", "group-home", "prison-or-jail", "retail-store", "grocery-market", "auto-service-station", "shopping-mall", "gas-station", "private", "hotel-or-motel", "dormitory", "boarding-house", "automobile", "airplane", "bus", "ferry", "ship-or-boat", "train", "motor-bike", "muni-mesh-network", "city-park", "rest-area", "traffic-control", "bus-stop", "kiosk"]
    hessid: str
    proxy_arp: Literal["enable", "disable"]
    l2tif: Literal["enable", "disable"]
    pame_bi: Literal["disable", "enable"]
    anqp_domain_id: int
    domain_name: str
    osu_ssid: str
    gas_comeback_delay: int
    gas_fragmentation_limit: int
    dgaf: Literal["enable", "disable"]
    deauth_request_timeout: int
    wnm_sleep_mode: Literal["enable", "disable"]
    bss_transition: Literal["enable", "disable"]
    venue_name: str
    venue_url: str
    roaming_consortium: str
    nai_realm: str
    oper_friendly_name: str
    oper_icon: str
    advice_of_charge: str
    osu_provider_nai: str
    terms_and_conditions: str
    osu_provider: str | list[str] | list[HsProfileOsuproviderItem]
    wan_metrics: str
    network_auth: str
    x3gpp_plmn: str
    conn_cap: str
    qos_map: str
    ip_addr_type: str
    wba_open_roaming: Literal["disable", "enable"]
    wba_financial_clearing_provider: str
    wba_data_clearing_provider: str
    wba_charging_currency: str
    wba_charging_rate: int


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class HsProfileResponse(TypedDict, total=False):
    """Response type for HsProfile - use with .dict property for typed dict access."""
    name: str
    release: int
    access_network_type: Literal["private-network", "private-network-with-guest-access", "chargeable-public-network", "free-public-network", "personal-device-network", "emergency-services-only-network", "test-or-experimental", "wildcard"]
    access_network_internet: Literal["enable", "disable"]
    access_network_asra: Literal["enable", "disable"]
    access_network_esr: Literal["enable", "disable"]
    access_network_uesa: Literal["enable", "disable"]
    venue_group: Literal["unspecified", "assembly", "business", "educational", "factory", "institutional", "mercantile", "residential", "storage", "utility", "vehicular", "outdoor"]
    venue_type: Literal["unspecified", "arena", "stadium", "passenger-terminal", "amphitheater", "amusement-park", "place-of-worship", "convention-center", "library", "museum", "restaurant", "theater", "bar", "coffee-shop", "zoo-or-aquarium", "emergency-center", "doctor-office", "bank", "fire-station", "police-station", "post-office", "professional-office", "research-facility", "attorney-office", "primary-school", "secondary-school", "university-or-college", "factory", "hospital", "long-term-care-facility", "rehab-center", "group-home", "prison-or-jail", "retail-store", "grocery-market", "auto-service-station", "shopping-mall", "gas-station", "private", "hotel-or-motel", "dormitory", "boarding-house", "automobile", "airplane", "bus", "ferry", "ship-or-boat", "train", "motor-bike", "muni-mesh-network", "city-park", "rest-area", "traffic-control", "bus-stop", "kiosk"]
    hessid: str
    proxy_arp: Literal["enable", "disable"]
    l2tif: Literal["enable", "disable"]
    pame_bi: Literal["disable", "enable"]
    anqp_domain_id: int
    domain_name: str
    osu_ssid: str
    gas_comeback_delay: int
    gas_fragmentation_limit: int
    dgaf: Literal["enable", "disable"]
    deauth_request_timeout: int
    wnm_sleep_mode: Literal["enable", "disable"]
    bss_transition: Literal["enable", "disable"]
    venue_name: str
    venue_url: str
    roaming_consortium: str
    nai_realm: str
    oper_friendly_name: str
    oper_icon: str
    advice_of_charge: str
    osu_provider_nai: str
    terms_and_conditions: str
    osu_provider: list[HsProfileOsuproviderItem]
    wan_metrics: str
    network_auth: str
    x3gpp_plmn: str
    conn_cap: str
    qos_map: str
    ip_addr_type: str
    wba_open_roaming: Literal["disable", "enable"]
    wba_financial_clearing_provider: str
    wba_data_clearing_provider: str
    wba_charging_currency: str
    wba_charging_rate: int


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class HsProfileOsuproviderItemObject(FortiObject[HsProfileOsuproviderItem]):
    """Typed object for osu-provider table items with attribute access."""
    name: str


class HsProfileObject(FortiObject):
    """Typed FortiObject for HsProfile with field access."""
    name: str
    release: int
    access_network_type: Literal["private-network", "private-network-with-guest-access", "chargeable-public-network", "free-public-network", "personal-device-network", "emergency-services-only-network", "test-or-experimental", "wildcard"]
    access_network_internet: Literal["enable", "disable"]
    access_network_asra: Literal["enable", "disable"]
    access_network_esr: Literal["enable", "disable"]
    access_network_uesa: Literal["enable", "disable"]
    venue_group: Literal["unspecified", "assembly", "business", "educational", "factory", "institutional", "mercantile", "residential", "storage", "utility", "vehicular", "outdoor"]
    venue_type: Literal["unspecified", "arena", "stadium", "passenger-terminal", "amphitheater", "amusement-park", "place-of-worship", "convention-center", "library", "museum", "restaurant", "theater", "bar", "coffee-shop", "zoo-or-aquarium", "emergency-center", "doctor-office", "bank", "fire-station", "police-station", "post-office", "professional-office", "research-facility", "attorney-office", "primary-school", "secondary-school", "university-or-college", "factory", "hospital", "long-term-care-facility", "rehab-center", "group-home", "prison-or-jail", "retail-store", "grocery-market", "auto-service-station", "shopping-mall", "gas-station", "private", "hotel-or-motel", "dormitory", "boarding-house", "automobile", "airplane", "bus", "ferry", "ship-or-boat", "train", "motor-bike", "muni-mesh-network", "city-park", "rest-area", "traffic-control", "bus-stop", "kiosk"]
    hessid: str
    proxy_arp: Literal["enable", "disable"]
    l2tif: Literal["enable", "disable"]
    pame_bi: Literal["disable", "enable"]
    anqp_domain_id: int
    domain_name: str
    osu_ssid: str
    gas_comeback_delay: int
    gas_fragmentation_limit: int
    dgaf: Literal["enable", "disable"]
    deauth_request_timeout: int
    wnm_sleep_mode: Literal["enable", "disable"]
    bss_transition: Literal["enable", "disable"]
    venue_name: str
    venue_url: str
    roaming_consortium: str
    nai_realm: str
    oper_friendly_name: str
    oper_icon: str
    advice_of_charge: str
    osu_provider_nai: str
    terms_and_conditions: str
    osu_provider: FortiObjectList[HsProfileOsuproviderItemObject]
    wan_metrics: str
    network_auth: str
    x3gpp_plmn: str
    conn_cap: str
    qos_map: str
    ip_addr_type: str
    wba_open_roaming: Literal["disable", "enable"]
    wba_financial_clearing_provider: str
    wba_data_clearing_provider: str
    wba_charging_currency: str
    wba_charging_rate: int


# ================================================================
# Main Endpoint Class
# ================================================================

class HsProfile:
    """
    
    Endpoint: wireless_controller/hotspot20/hs_profile
    Category: cmdb
    MKey: name
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    mkey: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # CMDB with mkey - overloads for single vs list returns
    @overload
    def get(
        self,
        name: str,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> HsProfileObject: ...
    
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObjectList[HsProfileObject]: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...

    # ================================================================
    # POST Method
    # ================================================================
    
    def post(
        self,
        payload_dict: HsProfilePayload | None = ...,
        name: str | None = ...,
        release: int | None = ...,
        access_network_type: Literal["private-network", "private-network-with-guest-access", "chargeable-public-network", "free-public-network", "personal-device-network", "emergency-services-only-network", "test-or-experimental", "wildcard"] | None = ...,
        access_network_internet: Literal["enable", "disable"] | None = ...,
        access_network_asra: Literal["enable", "disable"] | None = ...,
        access_network_esr: Literal["enable", "disable"] | None = ...,
        access_network_uesa: Literal["enable", "disable"] | None = ...,
        venue_group: Literal["unspecified", "assembly", "business", "educational", "factory", "institutional", "mercantile", "residential", "storage", "utility", "vehicular", "outdoor"] | None = ...,
        venue_type: Literal["unspecified", "arena", "stadium", "passenger-terminal", "amphitheater", "amusement-park", "place-of-worship", "convention-center", "library", "museum", "restaurant", "theater", "bar", "coffee-shop", "zoo-or-aquarium", "emergency-center", "doctor-office", "bank", "fire-station", "police-station", "post-office", "professional-office", "research-facility", "attorney-office", "primary-school", "secondary-school", "university-or-college", "factory", "hospital", "long-term-care-facility", "rehab-center", "group-home", "prison-or-jail", "retail-store", "grocery-market", "auto-service-station", "shopping-mall", "gas-station", "private", "hotel-or-motel", "dormitory", "boarding-house", "automobile", "airplane", "bus", "ferry", "ship-or-boat", "train", "motor-bike", "muni-mesh-network", "city-park", "rest-area", "traffic-control", "bus-stop", "kiosk"] | None = ...,
        hessid: str | None = ...,
        proxy_arp: Literal["enable", "disable"] | None = ...,
        l2tif: Literal["enable", "disable"] | None = ...,
        pame_bi: Literal["disable", "enable"] | None = ...,
        anqp_domain_id: int | None = ...,
        domain_name: str | None = ...,
        osu_ssid: str | None = ...,
        gas_comeback_delay: int | None = ...,
        gas_fragmentation_limit: int | None = ...,
        dgaf: Literal["enable", "disable"] | None = ...,
        deauth_request_timeout: int | None = ...,
        wnm_sleep_mode: Literal["enable", "disable"] | None = ...,
        bss_transition: Literal["enable", "disable"] | None = ...,
        venue_name: str | None = ...,
        venue_url: str | None = ...,
        roaming_consortium: str | None = ...,
        nai_realm: str | None = ...,
        oper_friendly_name: str | None = ...,
        oper_icon: str | None = ...,
        advice_of_charge: str | None = ...,
        osu_provider_nai: str | None = ...,
        terms_and_conditions: str | None = ...,
        osu_provider: str | list[str] | list[HsProfileOsuproviderItem] | None = ...,
        wan_metrics: str | None = ...,
        network_auth: str | None = ...,
        x3gpp_plmn: str | None = ...,
        conn_cap: str | None = ...,
        qos_map: str | None = ...,
        ip_addr_type: str | None = ...,
        wba_open_roaming: Literal["disable", "enable"] | None = ...,
        wba_financial_clearing_provider: str | None = ...,
        wba_data_clearing_provider: str | None = ...,
        wba_charging_currency: str | None = ...,
        wba_charging_rate: int | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> HsProfileObject: ...

    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: HsProfilePayload | None = ...,
        name: str | None = ...,
        release: int | None = ...,
        access_network_type: Literal["private-network", "private-network-with-guest-access", "chargeable-public-network", "free-public-network", "personal-device-network", "emergency-services-only-network", "test-or-experimental", "wildcard"] | None = ...,
        access_network_internet: Literal["enable", "disable"] | None = ...,
        access_network_asra: Literal["enable", "disable"] | None = ...,
        access_network_esr: Literal["enable", "disable"] | None = ...,
        access_network_uesa: Literal["enable", "disable"] | None = ...,
        venue_group: Literal["unspecified", "assembly", "business", "educational", "factory", "institutional", "mercantile", "residential", "storage", "utility", "vehicular", "outdoor"] | None = ...,
        venue_type: Literal["unspecified", "arena", "stadium", "passenger-terminal", "amphitheater", "amusement-park", "place-of-worship", "convention-center", "library", "museum", "restaurant", "theater", "bar", "coffee-shop", "zoo-or-aquarium", "emergency-center", "doctor-office", "bank", "fire-station", "police-station", "post-office", "professional-office", "research-facility", "attorney-office", "primary-school", "secondary-school", "university-or-college", "factory", "hospital", "long-term-care-facility", "rehab-center", "group-home", "prison-or-jail", "retail-store", "grocery-market", "auto-service-station", "shopping-mall", "gas-station", "private", "hotel-or-motel", "dormitory", "boarding-house", "automobile", "airplane", "bus", "ferry", "ship-or-boat", "train", "motor-bike", "muni-mesh-network", "city-park", "rest-area", "traffic-control", "bus-stop", "kiosk"] | None = ...,
        hessid: str | None = ...,
        proxy_arp: Literal["enable", "disable"] | None = ...,
        l2tif: Literal["enable", "disable"] | None = ...,
        pame_bi: Literal["disable", "enable"] | None = ...,
        anqp_domain_id: int | None = ...,
        domain_name: str | None = ...,
        osu_ssid: str | None = ...,
        gas_comeback_delay: int | None = ...,
        gas_fragmentation_limit: int | None = ...,
        dgaf: Literal["enable", "disable"] | None = ...,
        deauth_request_timeout: int | None = ...,
        wnm_sleep_mode: Literal["enable", "disable"] | None = ...,
        bss_transition: Literal["enable", "disable"] | None = ...,
        venue_name: str | None = ...,
        venue_url: str | None = ...,
        roaming_consortium: str | None = ...,
        nai_realm: str | None = ...,
        oper_friendly_name: str | None = ...,
        oper_icon: str | None = ...,
        advice_of_charge: str | None = ...,
        osu_provider_nai: str | None = ...,
        terms_and_conditions: str | None = ...,
        osu_provider: str | list[str] | list[HsProfileOsuproviderItem] | None = ...,
        wan_metrics: str | None = ...,
        network_auth: str | None = ...,
        x3gpp_plmn: str | None = ...,
        conn_cap: str | None = ...,
        qos_map: str | None = ...,
        ip_addr_type: str | None = ...,
        wba_open_roaming: Literal["disable", "enable"] | None = ...,
        wba_financial_clearing_provider: str | None = ...,
        wba_data_clearing_provider: str | None = ...,
        wba_charging_currency: str | None = ...,
        wba_charging_rate: int | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> HsProfileObject: ...

    # ================================================================
    # DELETE Method
    # ================================================================
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...

    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: HsProfilePayload | None = ...,
        name: str | None = ...,
        release: int | None = ...,
        access_network_type: Literal["private-network", "private-network-with-guest-access", "chargeable-public-network", "free-public-network", "personal-device-network", "emergency-services-only-network", "test-or-experimental", "wildcard"] | None = ...,
        access_network_internet: Literal["enable", "disable"] | None = ...,
        access_network_asra: Literal["enable", "disable"] | None = ...,
        access_network_esr: Literal["enable", "disable"] | None = ...,
        access_network_uesa: Literal["enable", "disable"] | None = ...,
        venue_group: Literal["unspecified", "assembly", "business", "educational", "factory", "institutional", "mercantile", "residential", "storage", "utility", "vehicular", "outdoor"] | None = ...,
        venue_type: Literal["unspecified", "arena", "stadium", "passenger-terminal", "amphitheater", "amusement-park", "place-of-worship", "convention-center", "library", "museum", "restaurant", "theater", "bar", "coffee-shop", "zoo-or-aquarium", "emergency-center", "doctor-office", "bank", "fire-station", "police-station", "post-office", "professional-office", "research-facility", "attorney-office", "primary-school", "secondary-school", "university-or-college", "factory", "hospital", "long-term-care-facility", "rehab-center", "group-home", "prison-or-jail", "retail-store", "grocery-market", "auto-service-station", "shopping-mall", "gas-station", "private", "hotel-or-motel", "dormitory", "boarding-house", "automobile", "airplane", "bus", "ferry", "ship-or-boat", "train", "motor-bike", "muni-mesh-network", "city-park", "rest-area", "traffic-control", "bus-stop", "kiosk"] | None = ...,
        hessid: str | None = ...,
        proxy_arp: Literal["enable", "disable"] | None = ...,
        l2tif: Literal["enable", "disable"] | None = ...,
        pame_bi: Literal["disable", "enable"] | None = ...,
        anqp_domain_id: int | None = ...,
        domain_name: str | None = ...,
        osu_ssid: str | None = ...,
        gas_comeback_delay: int | None = ...,
        gas_fragmentation_limit: int | None = ...,
        dgaf: Literal["enable", "disable"] | None = ...,
        deauth_request_timeout: int | None = ...,
        wnm_sleep_mode: Literal["enable", "disable"] | None = ...,
        bss_transition: Literal["enable", "disable"] | None = ...,
        venue_name: str | None = ...,
        venue_url: str | None = ...,
        roaming_consortium: str | None = ...,
        nai_realm: str | None = ...,
        oper_friendly_name: str | None = ...,
        oper_icon: str | None = ...,
        advice_of_charge: str | None = ...,
        osu_provider_nai: str | None = ...,
        terms_and_conditions: str | None = ...,
        osu_provider: str | list[str] | list[HsProfileOsuproviderItem] | None = ...,
        wan_metrics: str | None = ...,
        network_auth: str | None = ...,
        x3gpp_plmn: str | None = ...,
        conn_cap: str | None = ...,
        qos_map: str | None = ...,
        ip_addr_type: str | None = ...,
        wba_open_roaming: Literal["disable", "enable"] | None = ...,
        wba_financial_clearing_provider: str | None = ...,
        wba_data_clearing_provider: str | None = ...,
        wba_charging_currency: str | None = ...,
        wba_charging_rate: int | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject[Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject[Any]: ...
    
    @staticmethod
    def schema() -> FortiObject[Any]: ...


__all__ = [
    "HsProfile",
    "HsProfilePayload",
    "HsProfileResponse",
    "HsProfileObject",
]


""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: webfilter/profile
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
    overload,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class ProfileOverrideOvrdusergroupItem(TypedDict, total=False):
    """Nested item for override.ovrd-user-group field."""
    name: str


class ProfileOverrideProfileItem(TypedDict, total=False):
    """Nested item for override.profile field."""
    name: str


class ProfileWebKeywordmatchItem(TypedDict, total=False):
    """Nested item for web.keyword-match field."""
    pattern: str


class ProfileFtgdwfFiltersItem(TypedDict, total=False):
    """Nested item for ftgd-wf.filters field."""
    id: int
    category: int
    action: Literal["block", "authenticate", "monitor", "warning"]
    warn_duration: str
    auth_usr_grp: str | list[str]
    log: Literal["enable", "disable"]
    override_replacemsg: str
    warning_prompt: Literal["per-domain", "per-category"]
    warning_duration_type: Literal["session", "timeout"]


class ProfileFtgdwfRiskItem(TypedDict, total=False):
    """Nested item for ftgd-wf.risk field."""
    id: int
    risk_level: str
    action: Literal["block", "monitor"]
    log: Literal["enable", "disable"]


class ProfileFtgdwfQuotaItem(TypedDict, total=False):
    """Nested item for ftgd-wf.quota field."""
    id: int
    category: str | list[str]
    type: Literal["time", "traffic"]
    unit: Literal["B", "KB", "MB", "GB"]
    value: int
    duration: str
    override_replacemsg: str


class ProfileAntiphishInspectionentriesItem(TypedDict, total=False):
    """Nested item for antiphish.inspection-entries field."""
    name: str
    fortiguard_category: str | list[str]
    action: Literal["exempt", "log", "block"]


class ProfileAntiphishCustompatternsItem(TypedDict, total=False):
    """Nested item for antiphish.custom-patterns field."""
    pattern: str
    category: Literal["username", "password"]
    type: Literal["regex", "literal"]


class ProfileOverrideDict(TypedDict, total=False):
    """Nested object type for override field."""
    ovrd_cookie: Literal["allow", "deny"]
    ovrd_scope: Literal["user", "user-group", "ip", "browser", "ask"]
    profile_type: Literal["list", "radius"]
    ovrd_dur_mode: Literal["constant", "ask"]
    ovrd_dur: str
    profile_attribute: Literal["User-Name", "NAS-IP-Address", "Framed-IP-Address", "Framed-IP-Netmask", "Filter-Id", "Login-IP-Host", "Reply-Message", "Callback-Number", "Callback-Id", "Framed-Route", "Framed-IPX-Network", "Class", "Called-Station-Id", "Calling-Station-Id", "NAS-Identifier", "Proxy-State", "Login-LAT-Service", "Login-LAT-Node", "Login-LAT-Group", "Framed-AppleTalk-Zone", "Acct-Session-Id", "Acct-Multi-Session-Id"]
    ovrd_user_group: str | list[str] | list[ProfileOverrideOvrdusergroupItem]
    profile: str | list[str] | list[ProfileOverrideProfileItem]


class ProfileWebDict(TypedDict, total=False):
    """Nested object type for web field."""
    bword_threshold: int
    bword_table: int
    urlfilter_table: int
    content_header_list: int
    blocklist: Literal["enable", "disable"]
    allowlist: Literal["exempt-av", "exempt-webcontent", "exempt-activex-java-cookie", "exempt-dlp", "exempt-rangeblock", "extended-log-others"]
    safe_search: Literal["url", "header"]
    youtube_restrict: Literal["none", "strict", "moderate"]
    vimeo_restrict: str
    log_search: Literal["enable", "disable"]
    keyword_match: str | list[str] | list[ProfileWebKeywordmatchItem]


class ProfileFtgdwfDict(TypedDict, total=False):
    """Nested object type for ftgd-wf field."""
    options: Literal["error-allow", "rate-server-ip", "connect-request-bypass", "ftgd-disable"]
    exempt_quota: str | list[str]
    ovrd: str | list[str]
    filters: str | list[str] | list[ProfileFtgdwfFiltersItem]
    risk: str | list[str] | list[ProfileFtgdwfRiskItem]
    quota: str | list[str] | list[ProfileFtgdwfQuotaItem]
    max_quota_timeout: int
    rate_javascript_urls: Literal["disable", "enable"]
    rate_css_urls: Literal["disable", "enable"]
    rate_crl_urls: Literal["disable", "enable"]


class ProfileAntiphishDict(TypedDict, total=False):
    """Nested object type for antiphish field."""
    status: Literal["enable", "disable"]
    default_action: Literal["exempt", "log", "block"]
    check_uri: Literal["enable", "disable"]
    check_basic_auth: Literal["enable", "disable"]
    check_username_only: Literal["enable", "disable"]
    max_body_len: int
    inspection_entries: str | list[str] | list[ProfileAntiphishInspectionentriesItem]
    custom_patterns: str | list[str] | list[ProfileAntiphishCustompatternsItem]
    authentication: Literal["domain-controller", "ldap"]
    domain_controller: str
    ldap: str


class ProfileWispserversItem(TypedDict, total=False):
    """Nested item for wisp-servers field."""
    name: str


class ProfilePayload(TypedDict, total=False):
    """Payload type for Profile operations."""
    name: str
    comment: str
    feature_set: Literal["flow", "proxy"]
    replacemsg_group: str
    options: str | list[str]
    https_replacemsg: Literal["enable", "disable"]
    web_flow_log_encoding: Literal["utf-8", "punycode"]
    ovrd_perm: str | list[str]
    post_action: Literal["normal", "block"]
    override: ProfileOverrideDict
    web: ProfileWebDict
    ftgd_wf: ProfileFtgdwfDict
    antiphish: ProfileAntiphishDict
    wisp: Literal["enable", "disable"]
    wisp_servers: str | list[str] | list[ProfileWispserversItem]
    wisp_algorithm: Literal["primary-secondary", "round-robin", "auto-learning"]
    log_all_url: Literal["enable", "disable"]
    web_content_log: Literal["enable", "disable"]
    web_filter_activex_log: Literal["enable", "disable"]
    web_filter_command_block_log: Literal["enable", "disable"]
    web_filter_cookie_log: Literal["enable", "disable"]
    web_filter_applet_log: Literal["enable", "disable"]
    web_filter_jscript_log: Literal["enable", "disable"]
    web_filter_js_log: Literal["enable", "disable"]
    web_filter_vbs_log: Literal["enable", "disable"]
    web_filter_unknown_log: Literal["enable", "disable"]
    web_filter_referer_log: Literal["enable", "disable"]
    web_filter_cookie_removal_log: Literal["enable", "disable"]
    web_url_log: Literal["enable", "disable"]
    web_invalid_domain_log: Literal["enable", "disable"]
    web_ftgd_err_log: Literal["enable", "disable"]
    web_ftgd_quota_usage: Literal["enable", "disable"]
    extended_log: Literal["enable", "disable"]
    web_extended_all_action_log: Literal["enable", "disable"]
    web_antiphishing_log: Literal["enable", "disable"]


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class ProfileResponse(TypedDict, total=False):
    """Response type for Profile - use with .dict property for typed dict access."""
    name: str
    comment: str
    feature_set: Literal["flow", "proxy"]
    replacemsg_group: str
    options: str
    https_replacemsg: Literal["enable", "disable"]
    web_flow_log_encoding: Literal["utf-8", "punycode"]
    ovrd_perm: str
    post_action: Literal["normal", "block"]
    override: ProfileOverrideDict
    web: ProfileWebDict
    ftgd_wf: ProfileFtgdwfDict
    antiphish: ProfileAntiphishDict
    wisp: Literal["enable", "disable"]
    wisp_servers: list[ProfileWispserversItem]
    wisp_algorithm: Literal["primary-secondary", "round-robin", "auto-learning"]
    log_all_url: Literal["enable", "disable"]
    web_content_log: Literal["enable", "disable"]
    web_filter_activex_log: Literal["enable", "disable"]
    web_filter_command_block_log: Literal["enable", "disable"]
    web_filter_cookie_log: Literal["enable", "disable"]
    web_filter_applet_log: Literal["enable", "disable"]
    web_filter_jscript_log: Literal["enable", "disable"]
    web_filter_js_log: Literal["enable", "disable"]
    web_filter_vbs_log: Literal["enable", "disable"]
    web_filter_unknown_log: Literal["enable", "disable"]
    web_filter_referer_log: Literal["enable", "disable"]
    web_filter_cookie_removal_log: Literal["enable", "disable"]
    web_url_log: Literal["enable", "disable"]
    web_invalid_domain_log: Literal["enable", "disable"]
    web_ftgd_err_log: Literal["enable", "disable"]
    web_ftgd_quota_usage: Literal["enable", "disable"]
    extended_log: Literal["enable", "disable"]
    web_extended_all_action_log: Literal["enable", "disable"]
    web_antiphishing_log: Literal["enable", "disable"]


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class ProfileWispserversItemObject(FortiObject[ProfileWispserversItem]):
    """Typed object for wisp-servers table items with attribute access."""
    name: str


class ProfileOverrideOvrdusergroupItemObject(FortiObject[ProfileOverrideOvrdusergroupItem]):
    """Typed object for override.ovrd-user-group table items with attribute access."""
    name: str


class ProfileOverrideProfileItemObject(FortiObject[ProfileOverrideProfileItem]):
    """Typed object for override.profile table items with attribute access."""
    name: str


class ProfileWebKeywordmatchItemObject(FortiObject[ProfileWebKeywordmatchItem]):
    """Typed object for web.keyword-match table items with attribute access."""
    pattern: str


class ProfileFtgdwfFiltersItemObject(FortiObject[ProfileFtgdwfFiltersItem]):
    """Typed object for ftgd-wf.filters table items with attribute access."""
    id: int
    category: int
    action: Literal["block", "authenticate", "monitor", "warning"]
    warn_duration: str
    auth_usr_grp: str | list[str]
    log: Literal["enable", "disable"]
    override_replacemsg: str
    warning_prompt: Literal["per-domain", "per-category"]
    warning_duration_type: Literal["session", "timeout"]


class ProfileFtgdwfRiskItemObject(FortiObject[ProfileFtgdwfRiskItem]):
    """Typed object for ftgd-wf.risk table items with attribute access."""
    id: int
    risk_level: str
    action: Literal["block", "monitor"]
    log: Literal["enable", "disable"]


class ProfileFtgdwfQuotaItemObject(FortiObject[ProfileFtgdwfQuotaItem]):
    """Typed object for ftgd-wf.quota table items with attribute access."""
    id: int
    category: str | list[str]
    type: Literal["time", "traffic"]
    unit: Literal["B", "KB", "MB", "GB"]
    value: int
    duration: str
    override_replacemsg: str


class ProfileAntiphishInspectionentriesItemObject(FortiObject[ProfileAntiphishInspectionentriesItem]):
    """Typed object for antiphish.inspection-entries table items with attribute access."""
    name: str
    fortiguard_category: str | list[str]
    action: Literal["exempt", "log", "block"]


class ProfileAntiphishCustompatternsItemObject(FortiObject[ProfileAntiphishCustompatternsItem]):
    """Typed object for antiphish.custom-patterns table items with attribute access."""
    pattern: str
    category: Literal["username", "password"]
    type: Literal["regex", "literal"]


class ProfileOverrideObject(FortiObject):
    """Nested object for override field with attribute access."""
    ovrd_cookie: Literal["allow", "deny"]
    ovrd_scope: Literal["user", "user-group", "ip", "browser", "ask"]
    profile_type: Literal["list", "radius"]
    ovrd_dur_mode: Literal["constant", "ask"]
    ovrd_dur: str
    profile_attribute: Literal["User-Name", "NAS-IP-Address", "Framed-IP-Address", "Framed-IP-Netmask", "Filter-Id", "Login-IP-Host", "Reply-Message", "Callback-Number", "Callback-Id", "Framed-Route", "Framed-IPX-Network", "Class", "Called-Station-Id", "Calling-Station-Id", "NAS-Identifier", "Proxy-State", "Login-LAT-Service", "Login-LAT-Node", "Login-LAT-Group", "Framed-AppleTalk-Zone", "Acct-Session-Id", "Acct-Multi-Session-Id"]
    ovrd_user_group: str | list[str]
    profile: str | list[str]


class ProfileWebObject(FortiObject):
    """Nested object for web field with attribute access."""
    bword_threshold: int
    bword_table: int
    urlfilter_table: int
    content_header_list: int
    blocklist: Literal["enable", "disable"]
    allowlist: Literal["exempt-av", "exempt-webcontent", "exempt-activex-java-cookie", "exempt-dlp", "exempt-rangeblock", "extended-log-others"]
    safe_search: Literal["url", "header"]
    youtube_restrict: Literal["none", "strict", "moderate"]
    vimeo_restrict: str
    log_search: Literal["enable", "disable"]
    keyword_match: str | list[str]


class ProfileFtgdwfObject(FortiObject):
    """Nested object for ftgd-wf field with attribute access."""
    options: Literal["error-allow", "rate-server-ip", "connect-request-bypass", "ftgd-disable"]
    exempt_quota: str | list[str]
    ovrd: str | list[str]
    filters: str | list[str]
    risk: str | list[str]
    quota: str | list[str]
    max_quota_timeout: int
    rate_javascript_urls: Literal["disable", "enable"]
    rate_css_urls: Literal["disable", "enable"]
    rate_crl_urls: Literal["disable", "enable"]


class ProfileAntiphishObject(FortiObject):
    """Nested object for antiphish field with attribute access."""
    status: Literal["enable", "disable"]
    default_action: Literal["exempt", "log", "block"]
    check_uri: Literal["enable", "disable"]
    check_basic_auth: Literal["enable", "disable"]
    check_username_only: Literal["enable", "disable"]
    max_body_len: int
    inspection_entries: str | list[str]
    custom_patterns: str | list[str]
    authentication: Literal["domain-controller", "ldap"]
    domain_controller: str
    ldap: str


class ProfileObject(FortiObject):
    """Typed FortiObject for Profile with field access."""
    name: str
    comment: str
    feature_set: Literal["flow", "proxy"]
    replacemsg_group: str
    options: str
    https_replacemsg: Literal["enable", "disable"]
    web_flow_log_encoding: Literal["utf-8", "punycode"]
    ovrd_perm: str
    post_action: Literal["normal", "block"]
    override: ProfileOverrideObject
    web: ProfileWebObject
    ftgd_wf: ProfileFtgdwfObject
    antiphish: ProfileAntiphishObject
    wisp: Literal["enable", "disable"]
    wisp_servers: FortiObjectList[ProfileWispserversItemObject]
    wisp_algorithm: Literal["primary-secondary", "round-robin", "auto-learning"]
    log_all_url: Literal["enable", "disable"]
    web_content_log: Literal["enable", "disable"]
    web_filter_activex_log: Literal["enable", "disable"]
    web_filter_command_block_log: Literal["enable", "disable"]
    web_filter_cookie_log: Literal["enable", "disable"]
    web_filter_applet_log: Literal["enable", "disable"]
    web_filter_jscript_log: Literal["enable", "disable"]
    web_filter_js_log: Literal["enable", "disable"]
    web_filter_vbs_log: Literal["enable", "disable"]
    web_filter_unknown_log: Literal["enable", "disable"]
    web_filter_referer_log: Literal["enable", "disable"]
    web_filter_cookie_removal_log: Literal["enable", "disable"]
    web_url_log: Literal["enable", "disable"]
    web_invalid_domain_log: Literal["enable", "disable"]
    web_ftgd_err_log: Literal["enable", "disable"]
    web_ftgd_quota_usage: Literal["enable", "disable"]
    extended_log: Literal["enable", "disable"]
    web_extended_all_action_log: Literal["enable", "disable"]
    web_antiphishing_log: Literal["enable", "disable"]


# ================================================================
# Main Endpoint Class
# ================================================================

class Profile:
    """
    
    Endpoint: webfilter/profile
    Category: cmdb
    MKey: name
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    mkey: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # CMDB with mkey - overloads for single vs list returns
    @overload
    def get(
        self,
        name: str,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> ProfileObject: ...
    
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObjectList[ProfileObject]: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...

    # ================================================================
    # POST Method
    # ================================================================
    
    def post(
        self,
        payload_dict: ProfilePayload | None = ...,
        name: str | None = ...,
        comment: str | None = ...,
        feature_set: Literal["flow", "proxy"] | None = ...,
        replacemsg_group: str | None = ...,
        options: str | list[str] | None = ...,
        https_replacemsg: Literal["enable", "disable"] | None = ...,
        web_flow_log_encoding: Literal["utf-8", "punycode"] | None = ...,
        ovrd_perm: str | list[str] | None = ...,
        post_action: Literal["normal", "block"] | None = ...,
        override: ProfileOverrideDict | None = ...,
        web: ProfileWebDict | None = ...,
        ftgd_wf: ProfileFtgdwfDict | None = ...,
        antiphish: ProfileAntiphishDict | None = ...,
        wisp: Literal["enable", "disable"] | None = ...,
        wisp_servers: str | list[str] | list[ProfileWispserversItem] | None = ...,
        wisp_algorithm: Literal["primary-secondary", "round-robin", "auto-learning"] | None = ...,
        log_all_url: Literal["enable", "disable"] | None = ...,
        web_content_log: Literal["enable", "disable"] | None = ...,
        web_filter_activex_log: Literal["enable", "disable"] | None = ...,
        web_filter_command_block_log: Literal["enable", "disable"] | None = ...,
        web_filter_cookie_log: Literal["enable", "disable"] | None = ...,
        web_filter_applet_log: Literal["enable", "disable"] | None = ...,
        web_filter_jscript_log: Literal["enable", "disable"] | None = ...,
        web_filter_js_log: Literal["enable", "disable"] | None = ...,
        web_filter_vbs_log: Literal["enable", "disable"] | None = ...,
        web_filter_unknown_log: Literal["enable", "disable"] | None = ...,
        web_filter_referer_log: Literal["enable", "disable"] | None = ...,
        web_filter_cookie_removal_log: Literal["enable", "disable"] | None = ...,
        web_url_log: Literal["enable", "disable"] | None = ...,
        web_invalid_domain_log: Literal["enable", "disable"] | None = ...,
        web_ftgd_err_log: Literal["enable", "disable"] | None = ...,
        web_ftgd_quota_usage: Literal["enable", "disable"] | None = ...,
        extended_log: Literal["enable", "disable"] | None = ...,
        web_extended_all_action_log: Literal["enable", "disable"] | None = ...,
        web_antiphishing_log: Literal["enable", "disable"] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> ProfileObject: ...

    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: ProfilePayload | None = ...,
        name: str | None = ...,
        comment: str | None = ...,
        feature_set: Literal["flow", "proxy"] | None = ...,
        replacemsg_group: str | None = ...,
        options: str | list[str] | None = ...,
        https_replacemsg: Literal["enable", "disable"] | None = ...,
        web_flow_log_encoding: Literal["utf-8", "punycode"] | None = ...,
        ovrd_perm: str | list[str] | None = ...,
        post_action: Literal["normal", "block"] | None = ...,
        override: ProfileOverrideDict | None = ...,
        web: ProfileWebDict | None = ...,
        ftgd_wf: ProfileFtgdwfDict | None = ...,
        antiphish: ProfileAntiphishDict | None = ...,
        wisp: Literal["enable", "disable"] | None = ...,
        wisp_servers: str | list[str] | list[ProfileWispserversItem] | None = ...,
        wisp_algorithm: Literal["primary-secondary", "round-robin", "auto-learning"] | None = ...,
        log_all_url: Literal["enable", "disable"] | None = ...,
        web_content_log: Literal["enable", "disable"] | None = ...,
        web_filter_activex_log: Literal["enable", "disable"] | None = ...,
        web_filter_command_block_log: Literal["enable", "disable"] | None = ...,
        web_filter_cookie_log: Literal["enable", "disable"] | None = ...,
        web_filter_applet_log: Literal["enable", "disable"] | None = ...,
        web_filter_jscript_log: Literal["enable", "disable"] | None = ...,
        web_filter_js_log: Literal["enable", "disable"] | None = ...,
        web_filter_vbs_log: Literal["enable", "disable"] | None = ...,
        web_filter_unknown_log: Literal["enable", "disable"] | None = ...,
        web_filter_referer_log: Literal["enable", "disable"] | None = ...,
        web_filter_cookie_removal_log: Literal["enable", "disable"] | None = ...,
        web_url_log: Literal["enable", "disable"] | None = ...,
        web_invalid_domain_log: Literal["enable", "disable"] | None = ...,
        web_ftgd_err_log: Literal["enable", "disable"] | None = ...,
        web_ftgd_quota_usage: Literal["enable", "disable"] | None = ...,
        extended_log: Literal["enable", "disable"] | None = ...,
        web_extended_all_action_log: Literal["enable", "disable"] | None = ...,
        web_antiphishing_log: Literal["enable", "disable"] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> ProfileObject: ...

    # ================================================================
    # DELETE Method
    # ================================================================
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...

    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: ProfilePayload | None = ...,
        name: str | None = ...,
        comment: str | None = ...,
        feature_set: Literal["flow", "proxy"] | None = ...,
        replacemsg_group: str | None = ...,
        options: Literal["activexfilter", "cookiefilter", "javafilter", "block-invalid-url", "jscript", "js", "vbs", "unknown", "intrinsic", "wf-referer", "wf-cookie", "per-user-bal"] | list[str] | None = ...,
        https_replacemsg: Literal["enable", "disable"] | None = ...,
        web_flow_log_encoding: Literal["utf-8", "punycode"] | None = ...,
        ovrd_perm: Literal["bannedword-override", "urlfilter-override", "fortiguard-wf-override", "contenttype-check-override"] | list[str] | None = ...,
        post_action: Literal["normal", "block"] | None = ...,
        override: ProfileOverrideDict | None = ...,
        web: ProfileWebDict | None = ...,
        ftgd_wf: ProfileFtgdwfDict | None = ...,
        antiphish: ProfileAntiphishDict | None = ...,
        wisp: Literal["enable", "disable"] | None = ...,
        wisp_servers: str | list[str] | list[ProfileWispserversItem] | None = ...,
        wisp_algorithm: Literal["primary-secondary", "round-robin", "auto-learning"] | None = ...,
        log_all_url: Literal["enable", "disable"] | None = ...,
        web_content_log: Literal["enable", "disable"] | None = ...,
        web_filter_activex_log: Literal["enable", "disable"] | None = ...,
        web_filter_command_block_log: Literal["enable", "disable"] | None = ...,
        web_filter_cookie_log: Literal["enable", "disable"] | None = ...,
        web_filter_applet_log: Literal["enable", "disable"] | None = ...,
        web_filter_jscript_log: Literal["enable", "disable"] | None = ...,
        web_filter_js_log: Literal["enable", "disable"] | None = ...,
        web_filter_vbs_log: Literal["enable", "disable"] | None = ...,
        web_filter_unknown_log: Literal["enable", "disable"] | None = ...,
        web_filter_referer_log: Literal["enable", "disable"] | None = ...,
        web_filter_cookie_removal_log: Literal["enable", "disable"] | None = ...,
        web_url_log: Literal["enable", "disable"] | None = ...,
        web_invalid_domain_log: Literal["enable", "disable"] | None = ...,
        web_ftgd_err_log: Literal["enable", "disable"] | None = ...,
        web_ftgd_quota_usage: Literal["enable", "disable"] | None = ...,
        extended_log: Literal["enable", "disable"] | None = ...,
        web_extended_all_action_log: Literal["enable", "disable"] | None = ...,
        web_antiphishing_log: Literal["enable", "disable"] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject[Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject[Any]: ...
    
    @staticmethod
    def schema() -> FortiObject[Any]: ...


__all__ = [
    "Profile",
    "ProfilePayload",
    "ProfileResponse",
    "ProfileObject",
]


"""
Auto-generated normalization field sets from schema analysis.

Generated by: .dev/scripts/analyze_normalizable_fields.py
"""

# Simple array fields that use plain list format (not [{'name': '...'}])
# These should use normalize_to_string_list()
SIMPLE_ARRAY_FIELDS = {
    "admins",
    "commands",
    "config_ids",
    "devs",
    "dstintfrole",
    "endpoint_ip",
    "filters",
    "fortiextender-name",
    "fqdn",
    "guest",
    "hosts",
    "id_list",
    "ip",
    "ip_addresses",
    "macs",
    "mgmt_url_parameters",
    "mkeys",
    "objects",
    "orgunits",
    "policyid",
    "port_ranges",
    "ports",
    "qtypes",
    "report_types",
    "search_tables",
    "serials",
    "skip_tables",
    "srcintfrole",
    "tokens",
    "url",
    "user_group",
    "users",
    "uuid",
    "uuids",
}

# Table/complex fields with 'name' child that use [{'name': '...'}, ...] format
# These should use normalize_to_name_list()
NAME_LIST_FIELDS = {
}

# Multi-value option fields (space-separated strings)
# These should use normalize_day_field() or similar
MULTI_VALUE_OPTION_FIELDS = {
}



"""
Pydantic Models for CMDB - system/settings

Runtime validation models for system/settings configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class SettingsGuiDefaultPolicyColumns(BaseModel):
    """
    Child table model for gui-default-policy-columns.
    
    Default columns to display for policy lists on GUI.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str = Field(max_length=79, description="Select column name.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class SettingsVpnStatsLogEnum(str, Enum):
    """Allowed values for vpn_stats_log field."""
    IPSEC = "ipsec"
    PPTP = "pptp"
    L2TP = "l2tp"
    SSL = "ssl"

class SettingsV4EcmpModeEnum(str, Enum):
    """Allowed values for v4_ecmp_mode field."""
    SOURCE_IP_BASED = "source-ip-based"
    WEIGHT_BASED = "weight-based"
    USAGE_BASED = "usage-based"
    SOURCE_DEST_IP_BASED = "source-dest-ip-based"


# ============================================================================
# Main Model
# ============================================================================

class SettingsModel(BaseModel):
    """
    Pydantic model for system/settings configuration.
    
    Configure VDOM settings.
    
    Validation Rules:        - comments: max_length=255 pattern=        - vdom_type: pattern=        - lan_extension_controller_addr: max_length=255 pattern=        - lan_extension_controller_port: min=1024 max=65535 pattern=        - opmode: pattern=        - ngfw_mode: pattern=        - http_external_dest: pattern=        - firewall_session_dirty: pattern=        - manageip: pattern=        - gateway: pattern=        - ip: pattern=        - manageip6: pattern=        - gateway6: pattern=        - ip6: pattern=        - device: max_length=35 pattern=        - bfd: pattern=        - bfd_desired_min_tx: min=1 max=100000 pattern=        - bfd_required_min_rx: min=1 max=100000 pattern=        - bfd_detect_mult: min=1 max=50 pattern=        - bfd_dont_enforce_src_port: pattern=        - utf8_spam_tagging: pattern=        - wccp_cache_engine: pattern=        - vpn_stats_log: pattern=        - vpn_stats_period: min=0 max=4294967295 pattern=        - v4_ecmp_mode: pattern=        - mac_ttl: min=300 max=8640000 pattern=        - fw_session_hairpin: pattern=        - prp_trailer_action: pattern=        - snat_hairpin_traffic: pattern=        - dhcp_proxy: pattern=        - dhcp_proxy_interface_select_method: pattern=        - dhcp_proxy_interface: max_length=15 pattern=        - dhcp_proxy_vrf_select: min=0 max=511 pattern=        - dhcp_server_ip: pattern=        - dhcp6_server_ip: pattern=        - central_nat: pattern=        - gui_default_policy_columns: pattern=        - lldp_reception: pattern=        - lldp_transmission: pattern=        - link_down_access: pattern=        - nat46_generate_ipv6_fragment_header: pattern=        - nat46_force_ipv4_packet_forwarding: pattern=        - nat64_force_ipv6_packet_forwarding: pattern=        - detect_unknown_esp: pattern=        - intree_ses_best_route: pattern=        - auxiliary_session: pattern=        - asymroute: pattern=        - asymroute_icmp: pattern=        - tcp_session_without_syn: pattern=        - ses_denied_traffic: pattern=        - ses_denied_multicast_traffic: pattern=        - strict_src_check: pattern=        - allow_linkdown_path: pattern=        - asymroute6: pattern=        - asymroute6_icmp: pattern=        - sctp_session_without_init: pattern=        - sip_expectation: pattern=        - sip_nat_trace: pattern=        - h323_direct_model: pattern=        - status: pattern=        - sip_tcp_port: min=1 max=65535 pattern=        - sip_udp_port: min=1 max=65535 pattern=        - sip_ssl_port: min=0 max=65535 pattern=        - sccp_port: min=0 max=65535 pattern=        - multicast_forward: pattern=        - multicast_ttl_notchange: pattern=        - multicast_skip_policy: pattern=        - allow_subnet_overlap: pattern=        - deny_tcp_with_icmp: pattern=        - ecmp_max_paths: min=1 max=255 pattern=        - discovered_device_timeout: min=1 max=365 pattern=        - email_portal_check_dns: pattern=        - default_voip_alg_mode: pattern=        - gui_icap: pattern=        - gui_implicit_policy: pattern=        - gui_dns_database: pattern=        - gui_load_balance: pattern=        - gui_multicast_policy: pattern=        - gui_dos_policy: pattern=        - gui_object_colors: pattern=        - gui_route_tag_address_creation: pattern=        - gui_voip_profile: pattern=        - gui_ap_profile: pattern=        - gui_security_profile_group: pattern=        - gui_local_in_policy: pattern=        - gui_wanopt_cache: pattern=        - gui_explicit_proxy: pattern=        - gui_dynamic_routing: pattern=        - gui_sslvpn_personal_bookmarks: pattern=        - gui_sslvpn_realms: pattern=        - gui_policy_based_ipsec: pattern=        - gui_threat_weight: pattern=        - gui_spamfilter: pattern=        - gui_file_filter: pattern=        - gui_application_control: pattern=        - gui_ips: pattern=        - gui_dhcp_advanced: pattern=        - gui_vpn: pattern=        - gui_sslvpn: pattern=        - gui_wireless_controller: pattern=        - gui_advanced_wireless_features: pattern=        - gui_switch_controller: pattern=        - gui_fortiap_split_tunneling: pattern=        - gui_webfilter_advanced: pattern=        - gui_traffic_shaping: pattern=        - gui_wan_load_balancing: pattern=        - gui_antivirus: pattern=        - gui_webfilter: pattern=        - gui_videofilter: pattern=        - gui_dnsfilter: pattern=        - gui_waf_profile: pattern=        - gui_dlp_profile: pattern=        - gui_dlp_advanced: pattern=        - gui_virtual_patch_profile: pattern=        - gui_casb: pattern=        - gui_fortiextender_controller: pattern=        - gui_advanced_policy: pattern=        - gui_allow_unnamed_policy: pattern=        - gui_email_collection: pattern=        - gui_multiple_interface_policy: pattern=        - gui_policy_disclaimer: pattern=        - gui_ztna: pattern=        - gui_ot: pattern=        - gui_dynamic_device_os_id: pattern=        - gui_gtp: pattern=        - location_id: pattern=        - ike_session_resume: pattern=        - ike_quick_crash_detect: pattern=        - ike_dn_format: pattern=        - ike_port: min=1024 max=65535 pattern=        - ike_tcp_port: min=1 max=65535 pattern=        - ike_policy_route: pattern=        - ike_detailed_event_logs: pattern=        - block_land_attack: pattern=        - default_app_port_as_service: pattern=        - fqdn_session_check: pattern=        - ext_resource_session_check: pattern=        - dyn_addr_session_check: pattern=        - default_policy_expiry_days: min=0 max=365 pattern=        - gui_enforce_change_summary: pattern=        - internet_service_database_cache: pattern=        - internet_service_app_ctrl_size: min=0 max=4294967295 pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    comments: str | None = Field(max_length=255, default=None, description="VDOM comments.")    
    vdom_type: Literal["traffic", "lan-extension", "admin"] = Field(default="traffic", description="Vdom type (traffic, lan-extension or admin).")    
    lan_extension_controller_addr: str | None = Field(max_length=255, default=None, description="Controller IP address or FQDN to connect.")    
    lan_extension_controller_port: int | None = Field(ge=1024, le=65535, default=5246, description="Controller port to connect.")    
    opmode: Literal["nat", "transparent"] = Field(default="nat", description="Firewall operation mode (NAT or Transparent).")    
    ngfw_mode: Literal["profile-based", "policy-based"] | None = Field(default="profile-based", description="Next Generation Firewall (NGFW) mode.")    
    http_external_dest: Literal["fortiweb", "forticache"] | None = Field(default="fortiweb", description="Offload HTTP traffic to FortiWeb or FortiCache.")    
    firewall_session_dirty: Literal["check-all", "check-new", "check-policy-option"] | None = Field(default="check-all", description="Select how to manage sessions affected by firewall policy configuration changes.")    
    manageip: str = Field(description="Transparent mode IPv4 management IP address and netmask.")    
    gateway: str | None = Field(default="0.0.0.0", description="Transparent mode IPv4 default gateway IP address.")    
    ip: Any = Field(default="0.0.0.0 0.0.0.0", description="IP address and netmask.")    
    manageip6: str | None = Field(default="::/0", description="Transparent mode IPv6 management IP address and netmask.")    
    gateway6: str | None = Field(default="::", description="Transparent mode IPv6 default gateway IP address.")    
    ip6: str | None = Field(default="::/0", description="IPv6 address prefix for NAT mode.")    
    device: str = Field(max_length=35, description="Interface to use for management access for NAT mode.")  # datasource: ['system.interface.name']    
    bfd: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable Bi-directional Forwarding Detection (BFD) on all interfaces.")    
    bfd_desired_min_tx: int | None = Field(ge=1, le=100000, default=250, description="BFD desired minimal transmit interval (1 - 100000 ms, default = 250).")    
    bfd_required_min_rx: int | None = Field(ge=1, le=100000, default=250, description="BFD required minimal receive interval (1 - 100000 ms, default = 250).")    
    bfd_detect_mult: int | None = Field(ge=1, le=50, default=3, description="BFD detection multiplier (1 - 50, default = 3).")    
    bfd_dont_enforce_src_port: Literal["enable", "disable"] | None = Field(default="disable", description="Enable to not enforce verifying the source port of BFD Packets.")    
    utf8_spam_tagging: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable converting antispam tags to UTF-8 for better non-ASCII character support.")    
    wccp_cache_engine: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable WCCP cache engine.")    
    vpn_stats_log: list[SettingsVpnStatsLogEnum] = Field(default_factory=list, description="Enable/disable periodic VPN log statistics for one or more types of VPN. Separate names with a space.")    
    vpn_stats_period: int | None = Field(ge=0, le=4294967295, default=600, description="Period to send VPN log statistics (0 or 60 - 86400 sec).")    
    v4_ecmp_mode: SettingsV4EcmpModeEnum | None = Field(default=SettingsV4EcmpModeEnum.SOURCE_IP_BASED, description="IPv4 Equal-cost multi-path (ECMP) routing and load balancing mode.")    
    mac_ttl: int | None = Field(ge=300, le=8640000, default=300, description="Duration of MAC addresses in Transparent mode (300 - 8640000 sec, default = 300).")    
    fw_session_hairpin: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable checking for a matching policy each time hairpin traffic goes through the FortiGate.")    
    prp_trailer_action: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable action to take on PRP trailer.")    
    snat_hairpin_traffic: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable source NAT (SNAT) for VIP hairpin traffic.")    
    dhcp_proxy: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable the DHCP Proxy.")    
    dhcp_proxy_interface_select_method: Literal["auto", "sdwan", "specify"] | None = Field(default="auto", description="Specify how to select outgoing interface to reach server.")    
    dhcp_proxy_interface: str = Field(max_length=15, description="Specify outgoing interface to reach server.")  # datasource: ['system.interface.name']    
    dhcp_proxy_vrf_select: int | None = Field(ge=0, le=511, default=0, description="VRF ID used for connection to server.")    
    dhcp_server_ip: list[str] = Field(default_factory=list, description="DHCP Server IPv4 address.")    
    dhcp6_server_ip: list[str] = Field(default_factory=list, description="DHCPv6 server IPv6 address.")    
    central_nat: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable central NAT.")    
    gui_default_policy_columns: list[SettingsGuiDefaultPolicyColumns] = Field(default_factory=list, description="Default columns to display for policy lists on GUI.")    
    lldp_reception: Literal["enable", "disable", "global"] | None = Field(default="global", description="Enable/disable Link Layer Discovery Protocol (LLDP) reception for this VDOM or apply global settings to this VDOM.")    
    lldp_transmission: Literal["enable", "disable", "global"] | None = Field(default="global", description="Enable/disable Link Layer Discovery Protocol (LLDP) transmission for this VDOM or apply global settings to this VDOM.")    
    link_down_access: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable link down access traffic.")    
    nat46_generate_ipv6_fragment_header: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable NAT46 IPv6 fragment header generation.")    
    nat46_force_ipv4_packet_forwarding: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable mandatory IPv4 packet forwarding in NAT46.")    
    nat64_force_ipv6_packet_forwarding: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable mandatory IPv6 packet forwarding in NAT64.")    
    detect_unknown_esp: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable detection of unknown ESP packets (default = enable).")    
    intree_ses_best_route: Literal["force", "disable"] | None = Field(default="disable", description="Force the intree session to always use the best route.")    
    auxiliary_session: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable auxiliary session.")    
    asymroute: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable IPv4 asymmetric routing.")    
    asymroute_icmp: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable ICMP asymmetric routing.")    
    tcp_session_without_syn: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable allowing TCP session without SYN flags.")    
    ses_denied_traffic: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable including denied session in the session table.")    
    ses_denied_multicast_traffic: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable including denied multicast session in the session table.")    
    strict_src_check: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable strict source verification.")    
    allow_linkdown_path: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable link down path.")    
    asymroute6: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable asymmetric IPv6 routing.")    
    asymroute6_icmp: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable asymmetric ICMPv6 routing.")    
    sctp_session_without_init: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable SCTP session creation without SCTP INIT.")    
    sip_expectation: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable the SIP kernel session helper to create an expectation for port 5060.")    
    sip_nat_trace: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable recording the original SIP source IP address when NAT is used.")    
    h323_direct_model: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable H323 direct model.")    
    status: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable this VDOM.")    
    sip_tcp_port: list[int] = Field(ge=1, le=65535, default_factory=list, description="TCP port the SIP proxy monitors for SIP traffic (0 - 65535, default = 5060).")    
    sip_udp_port: list[int] = Field(ge=1, le=65535, default_factory=list, description="UDP port the SIP proxy monitors for SIP traffic (0 - 65535, default = 5060).")    
    sip_ssl_port: int | None = Field(ge=0, le=65535, default=5061, description="TCP port the SIP proxy monitors for SIP SSL/TLS traffic (0 - 65535, default = 5061).")    
    sccp_port: int | None = Field(ge=0, le=65535, default=2000, description="TCP port the SCCP proxy monitors for SCCP traffic (0 - 65535, default = 2000).")    
    multicast_forward: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable multicast forwarding.")    
    multicast_ttl_notchange: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable preventing the FortiGate from changing the TTL for forwarded multicast packets.")    
    multicast_skip_policy: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable allowing multicast traffic through the FortiGate without a policy check.")    
    allow_subnet_overlap: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable allowing interface subnets to use overlapping IP addresses.")    
    deny_tcp_with_icmp: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable denying TCP by sending an ICMP communication prohibited packet.")    
    ecmp_max_paths: int | None = Field(ge=1, le=255, default=255, description="Maximum number of Equal Cost Multi-Path (ECMP) next-hops. Set to 1 to disable ECMP routing (1 - 255, default = 255).")    
    discovered_device_timeout: int | None = Field(ge=1, le=365, default=28, description="Timeout for discovered devices (1 - 365 days, default = 28).")    
    email_portal_check_dns: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable using DNS to validate email addresses collected by a captive portal.")    
    default_voip_alg_mode: Literal["proxy-based", "kernel-helper-based"] | None = Field(default="proxy-based", description="Configure how the FortiGate handles VoIP traffic when a policy that accepts the traffic doesn't include a VoIP profile.")    
    gui_icap: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable ICAP on the GUI.")    
    gui_implicit_policy: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable implicit firewall policies on the GUI.")    
    gui_dns_database: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable DNS database settings on the GUI.")    
    gui_load_balance: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable server load balancing on the GUI.")    
    gui_multicast_policy: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable multicast firewall policies on the GUI.")    
    gui_dos_policy: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable DoS policies on the GUI.")    
    gui_object_colors: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable object colors on the GUI.")    
    gui_route_tag_address_creation: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable route-tag addresses on the GUI.")    
    gui_voip_profile: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable VoIP profiles on the GUI.")    
    gui_ap_profile: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable FortiAP profiles on the GUI.")    
    gui_security_profile_group: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable Security Profile Groups on the GUI.")    
    gui_local_in_policy: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable Local-In policies on the GUI.")    
    gui_wanopt_cache: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable WAN Optimization and Web Caching on the GUI.")    
    gui_explicit_proxy: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable the explicit proxy on the GUI.")    
    gui_dynamic_routing: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable dynamic routing on the GUI.")    
    gui_sslvpn_personal_bookmarks: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable SSL-VPN personal bookmark management on the GUI.")    
    gui_sslvpn_realms: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable SSL-VPN realms on the GUI.")    
    gui_policy_based_ipsec: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable policy-based IPsec VPN on the GUI.")    
    gui_threat_weight: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable threat weight on the GUI.")    
    gui_spamfilter: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable Antispam on the GUI.")    
    gui_file_filter: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable File-filter on the GUI.")    
    gui_application_control: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable application control on the GUI.")    
    gui_ips: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable IPS on the GUI.")    
    gui_dhcp_advanced: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable advanced DHCP options on the GUI.")    
    gui_vpn: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable IPsec VPN settings pages on the GUI.")    
    gui_sslvpn: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable SSL-VPN settings pages on the GUI.")    
    gui_wireless_controller: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable the wireless controller on the GUI.")    
    gui_advanced_wireless_features: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable advanced wireless features in GUI.")    
    gui_switch_controller: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable the switch controller on the GUI.")    
    gui_fortiap_split_tunneling: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable FortiAP split tunneling on the GUI.")    
    gui_webfilter_advanced: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable advanced web filtering on the GUI.")    
    gui_traffic_shaping: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable traffic shaping on the GUI.")    
    gui_wan_load_balancing: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable SD-WAN on the GUI.")    
    gui_antivirus: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable AntiVirus on the GUI.")    
    gui_webfilter: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable Web filtering on the GUI.")    
    gui_videofilter: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable Video filtering on the GUI.")    
    gui_dnsfilter: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable DNS Filtering on the GUI.")    
    gui_waf_profile: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable Web Application Firewall on the GUI.")    
    gui_dlp_profile: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable Data Loss Prevention on the GUI.")    
    gui_dlp_advanced: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable Show advanced DLP expressions on the GUI.")    
    gui_virtual_patch_profile: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable Virtual Patching on the GUI.")    
    gui_casb: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable Inline-CASB on the GUI.")    
    gui_fortiextender_controller: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable FortiExtender on the GUI.")    
    gui_advanced_policy: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable advanced policy configuration on the GUI.")    
    gui_allow_unnamed_policy: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable the requirement for policy naming on the GUI.")    
    gui_email_collection: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable email collection on the GUI.")    
    gui_multiple_interface_policy: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable adding multiple interfaces to a policy on the GUI.")    
    gui_policy_disclaimer: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable policy disclaimer on the GUI.")    
    gui_ztna: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable Zero Trust Network Access features on the GUI.")    
    gui_ot: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable Operational technology features on the GUI.")    
    gui_dynamic_device_os_id: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable Create dynamic addresses to manage known devices.")    
    gui_gtp: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable Manage general radio packet service (GPRS) protocols on the GUI.")    
    location_id: str | None = Field(default="0.0.0.0", description="Local location ID in the form of an IPv4 address.")    
    ike_session_resume: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable IKEv2 session resumption (RFC 5723).")    
    ike_quick_crash_detect: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable IKE quick crash detection (RFC 6290).")    
    ike_dn_format: Literal["with-space", "no-space"] | None = Field(default="with-space", description="Configure IKE ASN.1 Distinguished Name format conventions.")    
    ike_port: int | None = Field(ge=1024, le=65535, default=500, description="UDP port for IKE/IPsec traffic (default 500).")    
    ike_tcp_port: int | None = Field(ge=1, le=65535, default=443, description="TCP port for IKE/IPsec traffic (default 443).")    
    ike_policy_route: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable IKE Policy Based Routing (PBR).")    
    ike_detailed_event_logs: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable detail log for IKE events.")    
    block_land_attack: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable blocking of land attacks.")    
    default_app_port_as_service: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable policy service enforcement based on application default ports.")    
    fqdn_session_check: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable dirty session check caused by FQDN updates.")    
    ext_resource_session_check: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable dirty session check caused by external resource updates.")    
    dyn_addr_session_check: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable dirty session check caused by dynamic address updates.")    
    default_policy_expiry_days: int | None = Field(ge=0, le=365, default=30, description="Default policy expiry in days (0 - 365 days, default = 30).")    
    gui_enforce_change_summary: Literal["disable", "require", "optional"] | None = Field(default="require", description="Enforce change summaries for select tables in the GUI.")    
    internet_service_database_cache: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable Internet Service database caching.")    
    internet_service_app_ctrl_size: int | None = Field(ge=0, le=4294967295, default=32768, description="Maximum number of tuple entries (protocol, port, IP address, application ID) stored by the FortiGate unit (0 - 4294967295, default = 32768). A smaller value limits the FortiGate unit from learning about internet applications.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('device')
    @classmethod
    def validate_device(cls, v: Any) -> Any:
        """
        Validate device field.
        
        Datasource: ['system.interface.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('dhcp_proxy_interface')
    @classmethod
    def validate_dhcp_proxy_interface(cls, v: Any) -> Any:
        """
        Validate dhcp_proxy_interface field.
        
        Datasource: ['system.interface.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "SettingsModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_device_references(self, client: Any) -> list[str]:
        """
        Validate device references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/interface        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SettingsModel(
            ...     device="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_device_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.settings.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "device", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.interface.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Device '{value}' not found in "
                "system/interface"
            )        
        return errors    
    async def validate_dhcp_proxy_interface_references(self, client: Any) -> list[str]:
        """
        Validate dhcp_proxy_interface references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/interface        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SettingsModel(
            ...     dhcp_proxy_interface="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_dhcp_proxy_interface_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.settings.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "dhcp_proxy_interface", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.interface.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Dhcp-Proxy-Interface '{value}' not found in "
                "system/interface"
            )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_device_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_dhcp_proxy_interface_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "SettingsModel",    "SettingsGuiDefaultPolicyColumns",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.2
# Generated: 2026-01-21T16:24:00.668126Z
# ============================================================================


"""FortiOS CMDB - Vpn category"""

from . import ike
from . import ipsec
from . import ssl

__all__ = [
    "Ike",
    "Ipsec",
    "Ssl",
    "Vpn",
]


class Vpn:
    """Vpn endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Vpn endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.ike = ike.Ike(client)
        self.ipsec = ipsec.Ipsec(client)
        self.ssl = ssl.Ssl(client)



"""
Pydantic Models for CMDB - log/fortiguard/override_setting

Runtime validation models for log/fortiguard/override_setting configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class OverrideSettingUploadOptionEnum(str, Enum):
    """Allowed values for upload_option field."""
    STORE_AND_UPLOAD = "store-and-upload"
    REALTIME = "realtime"
    V_1_MINUTE = "1-minute"
    V_5_MINUTE = "5-minute"


# ============================================================================
# Main Model
# ============================================================================

class OverrideSettingModel(BaseModel):
    """
    Pydantic model for log/fortiguard/override_setting configuration.
    
    Override global FortiCloud logging settings for this VDOM.
    
    Validation Rules:        - override: pattern=        - status: pattern=        - upload_option: pattern=        - upload_interval: pattern=        - upload_day: pattern=        - upload_time: pattern=        - priority: pattern=        - max_log_rate: min=0 max=100000 pattern=        - access_config: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    override: Literal["enable", "disable"] | None = Field(default="disable", description="Overriding FortiCloud settings for this VDOM or use global settings.")    
    status: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable logging to FortiCloud.")    
    upload_option: OverrideSettingUploadOptionEnum | None = Field(default=OverrideSettingUploadOptionEnum.V_5_MINUTE, description="Configure how log messages are sent to FortiCloud.")    
    upload_interval: Literal["daily", "weekly", "monthly"] | None = Field(default="daily", description="Frequency of uploading log files to FortiCloud.")    
    upload_day: str | None = Field(default=None, description="Day of week to roll logs.")    
    upload_time: str | None = Field(default=None, description="Time of day to roll logs (hh:mm).")    
    priority: Literal["default", "low"] | None = Field(default="default", description="Set log transmission priority.")    
    max_log_rate: int | None = Field(ge=0, le=100000, default=0, description="FortiCloud maximum log rate in MBps (0 = unlimited).")    
    access_config: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable FortiCloud access to configuration and data.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "OverrideSettingModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "OverrideSettingModel",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.2
# Generated: 2026-01-21T16:24:01.220751Z
# ============================================================================


"""
Pydantic Models for CMDB - log/fortiguard/filter

Runtime validation models for log/fortiguard/filter configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Literal
from enum import Enum


# ============================================================================
# Child Table Models
# ============================================================================


class FilterFreeStyle(BaseModel):
    """
    Child table model for free-style.

    Free style filters.
    """

    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
    id: int | None = Field(ge=0, le=4294967295, default=0, description="Entry ID.")
    category: str = Field(default="traffic", description="Log category.")
    filter_: str = Field(max_length=1023, default="", description="Free style filter string.")
    filter_type: Literal["include", "exclude"] | None = Field(default="include", description="Include/exclude logs that match the filter.")

# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


class FilterSeverityEnum(str, Enum):
    """Allowed values for severity field."""
    EMERGENCY = "emergency"
    ALERT = "alert"
    CRITICAL = "critical"
    ERROR = "error"
    WARNING = "warning"
    NOTIFICATION = "notification"
    INFORMATION = "information"
    DEBUG = "debug"


# ============================================================================
# Main Model
# ============================================================================


class FilterModel(BaseModel):
    """
    Pydantic model for log/fortiguard/filter configuration.

    Filters for FortiCloud.

    Validation Rules:
        - severity: pattern=
        - forward_traffic: pattern=
        - local_traffic: pattern=
        - multicast_traffic: pattern=
        - sniffer_traffic: pattern=
        - ztna_traffic: pattern=
        - http_transaction: pattern=
        - anomaly: pattern=
        - voip: pattern=
        - gtp: pattern=
        - forti_switch: pattern=
        - free_style: pattern=
    """

    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names

    # ========================================================================
    # Model Fields
    # ========================================================================
    severity: str | FilterSeverityEnum | None = Field(default="information", description="Lowest severity level to log.")
    forward_traffic: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable forward traffic logging.")
    local_traffic: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable local in or out traffic logging.")
    multicast_traffic: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable multicast traffic logging.")
    sniffer_traffic: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable sniffer traffic logging.")
    ztna_traffic: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable ztna traffic logging.")
    http_transaction: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable log HTTP transaction messages.")
    anomaly: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable anomaly logging.")
    voip: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable VoIP logging.")
    gtp: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable GTP messages logging.")
    forti_switch: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable Forti-Switch logging.")
    free_style: list[FilterFreeStyle] | None = Field(default=None, description="Free style filters.")
    # ========================================================================
    # Custom Validators
    # ========================================================================

    # ========================================================================
    # Helper Methods
    # ========================================================================

    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.

        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)

    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "FilterModel":
        """
        Create model instance from FortiOS API response.

        Args:
            data: Response data from API

        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================


FilterModelDict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "FilterModel",    "FilterFreeStyle",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-14T21:59:52.101302Z
# ============================================================================



"""FortiOS CMDB - Ipsec category"""

from ..ipsec_base import Ipsec as IpsecBase
from .connection_count import ConnectionCount
from .tunnel_down import TunnelDown
from .tunnel_reset_stats import TunnelResetStats
from .tunnel_up import TunnelUp

__all__ = [
    "ConnectionCount",
    "Ipsec",
    "TunnelDown",
    "TunnelResetStats",
    "TunnelUp",
]


class Ipsec(IpsecBase):
    """Ipsec endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Ipsec endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        super().__init__(client)  # Initialize base class with GET methods
        self.connection_count = ConnectionCount(client)
        self.tunnel_down = TunnelDown(client)
        self.tunnel_reset_stats = TunnelResetStats(client)
        self.tunnel_up = TunnelUp(client)



"""FortiOS CMDB - EndpointControl category"""

from .fctems import Fctems
from .fctems_override import FctemsOverride
from .settings import Settings

__all__ = [
    "EndpointControl",
    "Fctems",
    "FctemsOverride",
    "Settings",
]


class EndpointControl:
    """EndpointControl endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """EndpointControl endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.fctems = Fctems(client)
        self.fctems_override = FctemsOverride(client)
        self.settings = Settings(client)



"""FortiOS CMDB - Log category"""

from . import disk
from . import fortianalyzer
from . import fortianalyzer2
from . import fortianalyzer3
from . import fortianalyzer_cloud
from . import fortiguard
from . import memory
from . import null_device
from . import syslogd
from . import syslogd2
from . import syslogd3
from . import syslogd4
from . import tacacs_plus_accounting
from . import tacacs_plus_accounting2
from . import tacacs_plus_accounting3
from . import webtrends
from .custom_field import CustomField
from .eventfilter import Eventfilter
from .gui_display import GuiDisplay
from .setting import Setting
from .threat_weight import ThreatWeight

__all__ = [
    "CustomField",
    "Disk",
    "Eventfilter",
    "Fortianalyzer",
    "Fortianalyzer2",
    "Fortianalyzer3",
    "FortianalyzerCloud",
    "Fortiguard",
    "GuiDisplay",
    "Log",
    "Memory",
    "NullDevice",
    "Setting",
    "Syslogd",
    "Syslogd2",
    "Syslogd3",
    "Syslogd4",
    "TacacsPlusAccounting",
    "TacacsPlusAccounting2",
    "TacacsPlusAccounting3",
    "ThreatWeight",
    "Webtrends",
]


class Log:
    """Log endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Log endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.disk = disk.Disk(client)
        self.fortianalyzer = fortianalyzer.Fortianalyzer(client)
        self.fortianalyzer2 = fortianalyzer2.Fortianalyzer2(client)
        self.fortianalyzer3 = fortianalyzer3.Fortianalyzer3(client)
        self.fortianalyzer_cloud = fortianalyzer_cloud.FortianalyzerCloud(client)
        self.fortiguard = fortiguard.Fortiguard(client)
        self.memory = memory.Memory(client)
        self.null_device = null_device.NullDevice(client)
        self.syslogd = syslogd.Syslogd(client)
        self.syslogd2 = syslogd2.Syslogd2(client)
        self.syslogd3 = syslogd3.Syslogd3(client)
        self.syslogd4 = syslogd4.Syslogd4(client)
        self.tacacs_plus_accounting = tacacs_plus_accounting.TacacsPlusAccounting(client)
        self.tacacs_plus_accounting2 = tacacs_plus_accounting2.TacacsPlusAccounting2(client)
        self.tacacs_plus_accounting3 = tacacs_plus_accounting3.TacacsPlusAccounting3(client)
        self.webtrends = webtrends.Webtrends(client)
        self.custom_field = CustomField(client)
        self.eventfilter = Eventfilter(client)
        self.gui_display = GuiDisplay(client)
        self.setting = Setting(client)
        self.threat_weight = ThreatWeight(client)



"""
Pydantic Models for MONITOR - fortiguard/service_communication_stats

Runtime validation models for fortiguard/service_communication_stats configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class ServiceCommunicationStatsServiceTypeEnum(str, Enum):
    """Allowed values for service_type field."""
    FORTICARE = "forticare"
    FORTIGUARD_DOWNLOAD = "fortiguard_download"
    FORTIGUARD_QUERY = "fortiguard_query"
    FORTICLOUD_LOG = "forticloud_log"
    FORTISANDBOX_CLOUD = "fortisandbox_cloud"
    FORTIGUARD_COM = "fortiguard.com"
    SDNS = "sdns"
    FORTITOKEN_REGISTRATION = "fortitoken_registration"
    SMS_SERVICE = "sms_service"


# ============================================================================
# Main Model
# ============================================================================

class ServiceCommunicationStatsModel(BaseModel):
    """
    Pydantic model for fortiguard/service_communication_stats configuration.
    
    Retrieve historical statistics for communication with FortiGuard services.
    
    Validation Rules:        - service_type: pattern=        - timeslot: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    service_type: ServiceCommunicationStatsServiceTypeEnum | None = Field(default=None)    
    timeslot: Literal["1_hour", "24_hour", "1_week"] | None = Field(default=None)    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "ServiceCommunicationStatsModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "ServiceCommunicationStatsModel",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-21T16:24:03.291615Z
# ============================================================================


"""Type stubs for EMAILFILTER category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .block_allow_list import BlockAllowList
    from .bword import Bword
    from .dnsbl import Dnsbl
    from .fortishield import Fortishield
    from .iptrust import Iptrust
    from .mheader import Mheader
    from .options import Options
    from .profile import Profile

__all__ = [
    "BlockAllowList",
    "Bword",
    "Dnsbl",
    "Fortishield",
    "Iptrust",
    "Mheader",
    "Options",
    "Profile",
    "Emailfilter",
]


class Emailfilter:
    """EMAILFILTER API category.
    
    All endpoints return FortiObject instances with:
    - Attribute access: response.field
    - Dictionary access: response["field"]
    - Convert to dict: response.dict or response.json
    """
    
    block_allow_list: BlockAllowList
    bword: Bword
    dnsbl: Dnsbl
    fortishield: Fortishield
    iptrust: Iptrust
    mheader: Mheader
    options: Options
    profile: Profile

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize emailfilter category with HTTP client."""
        ...



"""FortiOS CMDB - BotnetDomains category (stub)"""

from typing import Any
from ..botnet_domains_base import BotnetDomains as BotnetDomainsBase
from .hits import Hits
from .stat import Stat

class BotnetDomains(BotnetDomainsBase):
    """BotnetDomains endpoints wrapper for CMDB API."""

    hits: Hits
    stat: Stat

    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
